# Generated by Django 2.1 on 2019-10-16 14:55

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ripple', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='phone_number',
            field=models.CharField(max_length=20, null=True, validators=[django.core.validators.RegexValidator(message='Phone number format must be entered in the format: (+)9999999 with optional international code e.g (+4499999). Between 8 and 15 digits allowed.', regex='^\\+?1?\\d{9,15}$')]),
        ),
    ]
