# Generated by Django 3.0 on 2020-04-02 13:11

from django.db import migrations, models
import ripple.validators


class Migration(migrations.Migration):

    dependencies = [
        ('ripple', '0003_featureoption_featureurl'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='phone_number',
            field=models.CharField(max_length=20, null=True, validators=[ripple.validators.validate_gb_phone_number]),
        ),
    ]
